#!/bin/bash
# This script will configure an existing VALT server to function as a primary server. 
# You will need to run VALTStandbyServerConfig.sh on the warm standby server.

#standbyip="10.111.3.74"
#dbuser="ivsadmin"
#dbpass="admin51"
#slaveuser="ivs_user"
#slavepassword="admin51"

## Get Primary Server IP Address
function GetPrimaryServerIPAddress () {
	# Verify VALT ipaddress config file exists
	if [[ -s /usr/local/valt/conf/ipaddress ]]; then
		# set Primary Server IP Address
		primaryServerIPAddress=$(</usr/local/valt/conf/ipaddress)
		
		# Prompt user to confirm VALT ipaddress config file has correct ip address documented
		dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Configure Primary Server" --yesno "Please verify that the Primary Server's IP Address is:\n\n$primaryServerIPAddress" 8 50
		
		# User says YES to primary server ip address validation
		if [[ $? == 0 ]]; then		
			# do nothing and continue
			echo "User confirmed valid IP Address"
			hasValidNetworkConfig=true
			
		# User says NO to primary server ip address validation
		elif [[ $? == 1 ]]; then
			# Tell user to configure ip address through VALT menu
			dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Configure Primary Server" --msgbox "Please configure the correct IP Address in the VALT Menu before attempting to configure a Warm Standby Server setup." 8 50
			hasValidNetworkConfig=false
			echo "User denied valid IP Address"
		fi
		
	else
		# VALT primaryServerIPAddress config file does not exist; Tell user to configure ip address through VALT menu
		dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Configure Primary Server" --msgbox "IP Address configuration file not found.\n\nPlease configure the IP Address in the VALT Menu before attempting to configure a Warm Standby Server setup." 10 50

	fi
}

# Load VALT Environment Variables
function LoadEnvironmentVariables() {
	source /usr/local/valt/conf/valtpath
	source $valtpath/.env.local
}

# Update VALT Environment Variables
function UpdateEnvLocal () {
	name=$1
	value="${2//\//\\/}"
	if [[ ! -f $valtpath/.env.local ]]; then
		echo -e "$1=$2" > $valtpath/.env.local
	else
		sed -i -e '/^#\?\(\s*'"${name}"'\s*=\s*\).*/{s//\1'"${value}"'/;:a;n;ba;q}' -e '$a'"${name}"'='"${value}" $valtpath/.env.local
	fi
}

# Update Environment Variables with Warm Standby variables
function UpdateWarmStandbyVariables() {
	UpdateEnvLocal "IS_PRIMARY_SERVER" "$isPrimaryServer"
	UpdateEnvLocal "IS_STANDBY_SERVER" "$isStandbyServer"
	UpdateEnvLocal "PRIMARY_SERVER_IP_ADDRESS" "$primaryServerIPAddress"
	UpdateEnvLocal "STANDBY_SERVER_IP_ADDRESS" "$standbyServerIPAddress"
	UpdateEnvLocal "SLAVE_USER" "$slaveuser"
	UpdateEnvLocal "SLAVE_PASSWORD" "$slavepassword"
}


function ConfigurePrimaryServer() {
#	echo "WARNING: DO NOT RUN THIS SCRIPT MULTIPLE TIMES!!!"
	echo "You will need to run VALTStandbyServerConfig.sh on the warm standby server."
	echo "Press enter to continue."
	read r
	
	export NCURSES_NO_UTF8_ACS=1 
	
	# Must run script as Root
	if [[ $EUID -ne 0 ]]; then
		echo "You must run this script as root" 2>&1
		exit 1
	else
		# Initialize condition variables
		isPrimaryServer=false
		isStandbyServer=false
		slaveuser="ivs_user"
		slavepassword="admin51"
		
		hasValidNetworkConfig=true
		
		# Update .env.local with initialized Warm Standby Config
		UpdateWarmStandbyVariables
		
		# Load VALT Environment variables after updating them
		LoadEnvironmentVariables
		
		# Get IP Address
		#primaryServerIPAddress=$(</usr/local/valt/conf/ipaddress)
		GetPrimaryServerIPAddress
		
		# Verify Primary Server has valid configured IP Address
		if [[ $hasValidNetworkConfig == "true" ]]; then
			# Install Rsync
			apt install rsync
			
			# Configure "ivsadmin" user to not require password when running the commands "/usr/bin/rsync" and "/usr/bin/systemd-resolve"
			echo -e "ivsadmin ALL=(ALL) NOPASSWD: /usr/bin/rsync,/usr/bin/systemd-resolve" >> /etc/sudoers
			
			# Prompt for Standby Server IP
			standbyServerIPAddress=$(dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Configure Primary Server" --inputbox "Enter Standby Server IP Address" 8 50)

			# Update .env.local with Warm Standby Config's primary server and standby server's ip addresses
			UpdateWarmStandbyVariables
			
			# Load VALT Environment variables after updating them
			LoadEnvironmentVariables
			
			ssh-keygen -t rsa -f /root/.ssh/id_rsa -q -N ""
			ssh-copy-id -i /root/.ssh/id_rsa.pub ivsadmin@$STANDBY_SERVER_IP_ADDRESS
			
			# Comment out "bind-address" in MySQL config file
			sed -i "s|mysqlx-bind-address|#mysqlx-bind-address|g" /etc/mysql/mysql.conf.d/mysqld.cnf
			sed -i "s|bind-address|#bind-address|g" /etc/mysql/mysql.conf.d/mysqld.cnf
			
			# Add bind-address, server-id, log_bin, and binlog_do_db fields into MySQL config file
			echo -e "bind-address\t\t= $primaryServerIPAddress" >> /etc/mysql/mysql.conf.d/mysqld.cnf
			echo -e "server-id\t\t= 1" >> /etc/mysql/mysql.conf.d/mysqld.cnf
			echo -e "log_bin\t\t\t= /var/log/mysql/mysql-bin.log" >> /etc/mysql/mysql.conf.d/mysqld.cnf
		#	echo -e "binlog_do_db\t\t= v3" >> /etc/mysql/mysql.conf.d/mysqld.cnf
			echo -e "binlog_do_db\t\t= $DATABASE_NAME" >> /etc/mysql/mysql.conf.d/mysqld.cnf
			
			# Restart MySQL service
			service mysql restart
			
			# Update Environment Variables to verify Primary Standby Server IP Address is recorded as HOST
		#	sed -i "s|database_host: localhost|database_host: $primaryServerIPAddress|g" /var/www/v3/app/config/parameters.yml
		#	sed -i "s|database_host: localhost|database_host: $primaryServerIPAddress|g" $valtpath/.env.local
			
			# Open Firewall Rule over port 3306 to allow inbound traffic for MySQL 
			iptables -A INPUT -p tcp -m tcp --dport 3306 -j ACCEPT
			
			# Open Firewall Rule over port 22 to send outbound traffic for SSH
			iptables -A OUTPUT -p tcp -m tcp --dport 22 -j ACCEPT
			
			# Create a user to access the database from Standby Server
			# Grant Primary Server MySQL Database access to new user while connected to $STANDBY_SERVER_IP_ADDRESS
			mysql --defaults-extra-file=/etc/mysql/debian.cnf -e "CREATE USER '$DATABASE_USER'@'$STANDBY_SERVER_IP_ADDRESS' IDENTIFIED BY '$DATABASE_PASSWORD';"
			mysql --defaults-extra-file=/etc/mysql/debian.cnf -e "GRANT ALL PRIVILEGES ON *.* to '$DATABASE_USER'@'$STANDBY_SERVER_IP_ADDRESS';"
			
			mysql --defaults-extra-file=/etc/mysql/debian.cnf -e "CREATE USER '$SLAVE_USER'@'$STANDBY_SERVER_IP_ADDRESS' IDENTIFIED BY '$SLAVE_PASSWORD';"
			mysql --defaults-extra-file=/etc/mysql/debian.cnf -e "GRANT REPLICATION SLAVE ON *.* TO '$SLAVE_USER'@'$STANDBY_SERVER_IP_ADDRESS';"
			mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf -e "FLUSH PRIVILEGES;"
			
			# Go into $DATABASE_NAME database and clear all tables that are "read locked"
		#	mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf -e "USE v3;"
			mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf -e "USE $DATABASE_NAME;"
			mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf -e "FLUSH TABLES WITH READ LOCK;"
			
			# Get File value from $DATABASE_NAME database
		#	tmp1=$(mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf v3 -e "SHOW MASTER STATUS\G;" | grep File)
			tmp1=$(mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf $DATABASE_NAME -e "SHOW MASTER STATUS\G;" | grep File)
			sqlfile=${tmp1#*:} 
			
			# Remove whitespaces in variable
			sqlfile=`echo $sqlfile | sed -e 's/^[[:space:]]*//'`
			
			# Get Position value from $DATABASE_NAME database
		#	tmp1=$(mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf v3 -e "SHOW MASTER STATUS\G;" | grep Position)
			tmp1=$(mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf $DATABASE_NAME -e "SHOW MASTER STATUS\G;" | grep Position)
			position=${tmp1#*:}
			
			# Remove whitespaces in variable
			position=`echo $position | sed -e 's/^[[:space:]]*//'`
			
			###I suspect this is not necessary
		#	sqlfile=$(echo $sqlfile | xargs)
			
			# Create backup of $DATABASE_NAME database
		#	mysqldump --defaults-extra-file=/usr/local/valt/conf/sql.cnf --opt v3 > /usr/local/valt/backup/v3_sync.sql
			mysqldump --defaults-extra-file=/usr/local/valt/conf/sql.cnf --opt $DATABASE_NAME > /usr/local/valt/backup/valt_sync.sql
			mysql --defaults-extra-file=/usr/local/valt/conf/sql.cnf -e "UNLOCK TABLES;"
			
			# Save Firewall rules
			netfilter-persistent save
			
			# Remove whitespaces from these variables
			
			
			
			# Create configuration file to send to Failover Standby Server
			# Read out Master Log File and Master Log Position because this is required to configure the Failover  Server
			echo "masterlogfile=$sqlfile" > /usr/local/valt/conf/primaryserver
			echo "masterlogposition=$position" >> /usr/local/valt/conf/primaryserver
			echo "primaryserveripaddress=$PRIMARY_SERVER_IP_ADDRESS" >> /usr/local/valt/conf/primaryserver
			echo "standbyserveripaddress=$STANDBY_SERVER_IP_ADDRESS" >> /usr/local/valt/conf/primaryserver
			
			# Write out SSL Status of Primary Standby Server to config file
			if [[ -s /usr/local/valt/conf/ssl ]]; then
				echo "sslenabled=true" >> /usr/local/valt/conf/primaryserver
				
				# SSLType is equal to first line of the ssl config file
				ssltype=$(sed -n '1p;q' /usr/local/valt/conf/ssl)
				echo "ssltype=$ssltype" >> /usr/local/valt/conf/primaryserver
				
			elif [[ ! -s /usr/local/valt/conf/ssl ]]; then
				echo "sslenabled=false" >> /usr/local/valt/conf/primaryserver
			fi
			
			###I am getting stuck trying to SCP and RSYNC files over "Could not resolve hostname" "lost connection"
			
			LoadEnvironmentVariables
			
			# Send PrimaryServer config file over to standbyserver
			scp -i /root/.ssh/id_rsa.pub /usr/local/valt/conf/primaryserver ivsadmin@$STANDBY_SERVER_IP_ADDRESS:/tmp
			
			# Use rsync instead of scp because rsync keeps symlinks intact
			rsync -av --progress /etc/letsencrypt ivsadmin@$STANDBY_SERVER_IP_ADDRESS:/tmp
			
			# Copy over VALT scripts
			scp -r /usr/local/valt/bin ivsadmin@$STANDBY_SERVER_IP_ADDRESS:/tmp
			
			scp /usr/local/valt/conf/ssl ivsadmin@$STANDBY_SERVER_IP_ADDRESS:/tmp
			
			# Send the Warm Standby Config package to the standby server
			scp /home/ivsadmin/VALTRedundancy.tar.gz ivsadmin@$STANDBY_SERVER_IP_ADDRESS:/home/ivsadmin/
			
			# Primary Standby Server configured successfully
			isPrimaryServer=true
			
			UpdateWarmStandbyVariables
			
			dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Configure Primary Server" --msgbox "Primary Server configured successfully. Proceed to configure the Standby Server." 6 50
			
		else
			dialog --output-fd 1 --nocancel --clear --backtitle "VALT Warm Standby Server Configuration" --title "Configure Primary Server" --msgbox "Primary Server Configuration cancelled." 6 50
		fi
	fi
}

# Load environment variables
LoadEnvironmentVariables

# Configure Priamry Server
ConfigurePrimaryServer

